/*
 * Copyright (C) 2015-2017 Alibaba Group Holding Limited
 */

#include <stdio.h>

#include <aos/kernel.h>
#include "aos/cli.h"

#include "espressif/esp8266/pin_mux_register.h"
#include "espressif/esp_common.h"
#include "espressif/pwm.h"
#include "pwm.h"

#include "linkkit/dev_model_api.h"
#include "linkkit/infra/infra_compat.h"
#include "linkkit/infra/infra_config.h"
#include "linkkit/infra/infra_defs.h"
#include "linkkit/infra/infra_types.h"
#include "linkkit/wrappers/wrappers.h"

#include "cJSON.h"
#include "linkkit_client.h"


#define PRODUCT_KEY "<ProductKey>"
#define PRODUCT_SECRET "<ProductSecret>"
#define DEVICE_NAME "<DeviceName>"
#define DEVICE_SECRET "<DeviceSecret>"


#define WIFI_SSID "<WIFI_SSID>"
#define WIFI_PASSWD "<WIFI_PASSWD>"

/* pin:12  ->  B
 * pin:13  ->  G
 * pin:15  ->  R */
static void led_rgb_set(uint8_t r, uint8_t g, uint8_t b)
{
    pwm_set_duty(g * 1000 / 255, 0);
    pwm_set_duty(b * 1000 / 255, 1);
    pwm_set_duty(r * 1000 / 255, 2);
    pwm_start();
}

/* pwm init */
static void led_rgb_init(void)
{
    uint32_t pwm_duty_init[3] = {0};

    printf("led_rgb_init...\r\n");
    uint32_t io_info[][3] = {
        {PERIPHS_IO_MUX_MTDI_U, FUNC_GPIO12, 13},
        {PERIPHS_IO_MUX_MTCK_U, FUNC_GPIO13, 12},
        {PERIPHS_IO_MUX_MTDO_U, FUNC_GPIO15, 15},
    };

    /* start init */
    pwm_init(1000, pwm_duty_init, 3, io_info);
    led_rgb_set(0, 0, 0);
}

/* parse json-data received from IoT platform  */
static int user_property_set_event_handler(const int devid, const char *request,
                                           const int request_len)
{
    cJSON *root = NULL;
    cJSON *rgb_color = NULL;
    uint8_t red = 0, green = 0, blue = 0;

    printf("Property Set Received, Devid: %d, Request: %s\r\n", devid, request);
    if ((root = cJSON_Parse(request)) == NULL)
    {
        printf("json parse error\r\n");
        return -1;
    }

    if ((rgb_color = cJSON_GetObjectItem(root, "RGBColor")) == NULL)
    {
        printf("Not found RGBColor parameter\r\n");
        return 0;
    }

    red = cJSON_GetObjectItem(rgb_color, "Red")->valueint;
    green = cJSON_GetObjectItem(rgb_color, "Green")->valueint;
    blue = cJSON_GetObjectItem(rgb_color, "Blue")->valueint;

    printf("red:%d, green:%d, blue:%d\r\n", red, green, blue);
    led_rgb_set(red, green, blue);
    linkkit_set_props((unsigned char *)request, request_len);

    aos_free(root);
    return 0;
}

/* IoT platform connected callback */
int linkkit_connect_ok(void)
{
    int ret = -1;
    unsigned char *default_rgbcolor =
        "{\"RGBColor\":{\"Red\":255,\"Green\":0,\"Blue\":0}}";

    led_rgb_set(255, 0, 0);
    linkkit_set_props(default_rgbcolor, strlen(default_rgbcolor));
    return 0;
}

/* application init, including gpio, wifi. */
void app_init(void)
{
    led_rgb_init();
    wifi_start(WIFI_SSID, WIFI_PASSWD);
}

/* start connecting to IoT platform */
void app_start(void)
{
    led_rgb_set(0, 0, 255);
    linkkit_connect(PRODUCT_KEY, DEVICE_NAME, DEVICE_SECRET, PRODUCT_SECRET,
                    user_property_set_event_handler, linkkit_connect_ok);

    linkkit_start();
}